{% load i18n sizeformat parse_date %}
{% load url from future %}

<div class="info row detail">
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Name" %}</dt>
    <dd>{{ policyaction.name|default:_("-") }}</dd>

    <dt>{% trans "Description" %}</dt>
    <dd>{{ policyaction.description|default:_("-") }}</dd>

    <dt>{% trans "ID" %}</dt>
    <dd>{{ policyaction.id }}</dd>

    <dt>{% trans "Project ID" %}</dt>
    <dd>{{ policyaction.tenant_id }}</dd>
	
	<dt>{% trans "Action Value" %}</dt>
    <dd>{{ policyaction.action_value|default:_("-") }}</dd>

    <dt>{% trans "Action Type" %}</dt>
    <dd>{{ policyaction.action_type|default:_("-") }}</dd>
    
  </dl>
</div>
