{% load i18n sizeformat parse_date %}
{% load url from future %}

<div class="info row detail">
	<hr class="header_rule">
	<dl>
		<dt>{% trans "Name" %}</dt>
		<dd>{{ policy.name|default:_("-") }}</dd>

		<dt>{% trans "Description" %}</dt>
		<dd>{{ policy.description|default:_("-") }}</dd>

		<dt>{% trans "ID" %}</dt>
		<dd>{{ policy.id }} </dd>
		<dt>{% trans "Network Service Params" %}</dt>
		<dd>{{ policy.network_service_params }} </dd>
	</dl>
</div>
   
