{% load i18n sizeformat parse_date %}
{% load url from future %}

<div class="info row detail">
	<hr class="header_rule">
	<dl>
		<dt>{% trans "Name" %}</dt>
		<dd>{{ policy_target.name|default:_("-") }}</dd>

		<dt>{% trans "Description" %}</dt>
		<dd>{{ policy_target.description|default:_("-") }}</dd>

		<dt>{% trans "ID" %}</dt>
		<dd>{{ policy_target.id }} </dd>

		<dt>{% trans "Project ID" %}</dt>
		<dd>{{ policy_target.tenant_id }}</dd>

                <dt>{% trans "Network Service Policy" %}</dt>
                <dd>{% if nsp %}
                        <a href="{% url 'horizon:project:network_policy:service_policy_details' nsp.id %}">{{nsp.name}}</a>
                    {% else %}
                        {{ _("-") }}
                    {% endif %}
                </dd>
	</dl>
	{% if l3_policy %}
	<div class='panel panel-default'>
		<div class='panel-heading'>{% trans "L3 Policies" %}</div>
		<div class='panel-body'>
			<ul class="unstyled">
				<li><a href="{% url 'horizon:project:network_policy:l3policy_details' l3_policy.id %}">{{l3_policy.name}}:{{l3_policy.id}}</a></li>
			</ul>
		</div>
	</div>
	{% endif %}

        {% if l2_policy %}
	<div class='panel panel-default'>
		<div class='panel-heading'>{% trans "L2 Policies" %}</div>
		<div class='panel-body'>
			<ul class="unstyled">
				<li><a href="{% url 'horizon:project:network_policy:l2policy_details' l2_policy.id %}">{{l2_policy.name}}:{{l2_policy.id}}</a></li>
			</ul>
		</div>
	</div>
	{% endif %}
</div>

