{% load i18n sizeformat parse_date %}
{% load url from future %}

<div class="info row detail">
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Name" %}</dt>
    <dd>{{ policyrule.name|default:_("-") }}</dd>

    <dt>{% trans "Description" %}</dt>
    <dd>{{ policyrule.description|default:_("-") }}</dd>

    <dt>{% trans "ID" %}</dt>
    <dd>{{ policyrule.id }}</dd>

    <dt>{% trans "Project ID" %}</dt>
    <dd>{{ policyrule.tenant_id }}</dd>
	
	<dt>{% trans "Enabled" %}</dt>
    <dd>{{ policyrule.enabled|default:_("-") }}</dd>
    
    <dt>{% trans "Policy Actions" %}</dt>
    <dd>
      {% for action in actions%}
      {% url 'horizon:project:application_policy:policyactiondetails' action.id as action_url %}
       <a href="{{ action_url }}"> {{action.name}}:{{ action.id }}</a><br>
      {% endfor %}
    </dd>
    <dt>{% trans "Policy Classifier" %}</dt>
    <dd>
      {% for classifier in classifiers %}
      
      {% url 'horizon:project:application_policy:policyclassifierdetails' classifier.id as classifier_url %}
      <a href="{{ classifier_url }}"> {{classifier.name}}:{{ classifier.id }}</a><br>
      {% endfor %}
    </dd>
  </dl>
</div>
